@@ -49,7 +49,7 @@ get_beval_info(
 	col = X_2_COL(beval->x);
     }
 #endif
-    wp = mouse_find_win(&row, &col);
+    wp = mouse_find_win(&row, &col, FAIL_POPUP);
     if (wp != NULL && row >= 0 && row < wp->w_height && col < wp->w_width)
     {
 	/* Found a window and the cursor is in the text.  Now find the line
@@ -141,6 +141,7 @@ get_beval_info(
 
 /*
  * Show a balloon with "mesg" or "list".
+ * Hide the balloon when both are NULL.
  */
     void
 post_balloon(BalloonEval *beval UNUSED, char_u *mesg, list_T *list UNUSED)
@@ -153,7 +154,7 @@ post_balloon(BalloonEval *beval UNUSED, char_u *mesg, list_T *list UNUSED)
 # endif
 # ifdef FEAT_BEVAL_GUI
     if (gui.in_use)
-	/* GUI can't handle a list */
+	// GUI can't handle a list
 	gui_mch_post_balloon(beval, mesg);
 # endif
 }
