@@ -15,11 +15,7 @@
 
 #if defined(FEAT_DIGRAPHS) || defined(PROTO)
 
-#ifdef FEAT_MBYTE
 typedef int result_T;
-#else
-typedef char_u result_T;
-#endif
 
 typedef struct digraph
 {
@@ -28,8 +24,7 @@ typedef struct digraph
     result_T	result;
 } digr_T;
 
-static int getexactdigraph(int, int, int);
-static void printdigraph(digr_T *);
+static void printdigraph(digr_T *dp, result_T *previous);
 
 /* digraphs added by the user */
 static garray_T	user_digraphs = {0, 0, (int)sizeof(digr_T), 10, NULL};
@@ -39,8 +34,8 @@ static garray_T	user_digraphs = {0, 0, (int)sizeof(digr_T), 10, NULL};
  * compilers cannot handle them (Amiga SAS/C is the most picky one).
  */
 static digr_T digraphdefault[] =
-#ifdef __MINT__
 
+#ifdef __MINT__
 	/*
 	 * ATARI digraphs
 	 */
@@ -321,137 +316,7 @@ static digr_T digraphdefault[] =
 	};
 
 #  else
-#   if defined(MACOS_X) && !defined(FEAT_MBYTE)
-
-	/*
-	 * Macintosh digraphs
-	 */
-       {{'a', 't', 64},		/* @ */
-	{'A', '"', 128},	/* ~@ XX */
-	{'A', 'o', 129},	/*  */
-	{'C', ',', 130},	/*  */
-	{'E', '\'', 131},	/*  */
-	{'N', '~', 132},	/*  */
-	{'O', '"', 133},	/*  */
-	{'U', '"', 134},	/*  */
-	{'a', '\'', 135},	/* ~G XX */
-	{'a', '`', 136},	/* ~H XX */
-	{'a', '^', 137},	/*  */
-	{'a', '"', 138},	/*  */
-	{'a', '~', 139},	/*  */
-	{'a', 'o', 140},	/*  */
-	{'c', ',', 141},	/*  */
-	{'e', '\'', 142},	/*  */
-	{'e', '`', 143},	/*  */
-	{'e', '^', 144},	/*  */
-	{'e', '"', 145},	/*  */
-	{'i', '\'', 146},	/*  */
-	{'i', '`', 147},	/*  */
-	{'i', '^', 148},	/*  */
-	{'i', '"', 149},	/*  */
-	{'n', '~', 150},	/*  */
-	{'o', '\'', 151},	/*  */
-	{'o', '`', 152},	/*  */
-	{'o', '^', 153},	/*  */
-	{'o', '"', 154},	/*  */
-	{'o', '~', 155},	/* o */
-	{'u', '\'', 156},	/*  */
-	{'u', '`', 157},	/* ~] XX */
-	{'u', '^', 158},	/*  */
-	{'u', '"', 159},	/*  */
-	{'+', '_', 160},	/*  */
-	{'~', 'o', 161},	/*  */
-	{'c', '|', 162},	/*  */
-	{'$', '$', 163},	/*  */
-	{'p', 'a', 164},	/*  */
-	{'.', '.', 165},	/* * */
-	{'P', 'P', 166},	/*  */
-	{'s', 's', 167},	/*  */
-	{'r', 'O', 168},	/*  */
-	{'c', 'O', 169},	/*  */
-	{'T', 'M', 170},	/*  */
-	{'=', '/', 173},	/*  */
-	{'A', 'E', 174},	/*  */
-	{'O', '/', 175},	/*  */
-	{'0', '0', 176},	/*  */
-	{'+', '-', 177},	/*  */
-	{'<', '=', 178},	/*  */
-	{'>', '=', 179},	/*  */
-	{'Y', '-', 180},	/*  */
-	{'j', 'u', 181},	/*  */
-	{'m', 'u', 181},	/*  */
-	{'d', 'd', 182},	/*  */
-	{'S', 'S', 183},	/*  */
-	{'S', 'I', 183},	/*  */
-	{'P', 'I', 184},	/*  */
-	{'p', 'i', 185},	/*  */
-	{'I', 'I', 186},	/*  */
-	{'a', '-', 187},	/*  */
-	{'o', '-', 188},	/*  */
-	{'O', 'M', 189},	/*  */
-	{'a', 'e', 190},	/*  */
-	{'o', '/', 191},	/*  */
-	{'~', '?', 192},	/*  */
-	{'~', '!', 193},	/*  */
-	{'-', ',', 194},	/*  */
-	{'v', '-', 195},	/* ~H XX */
-	{'f', '-', 196},	/*  */
-	{'~', '~', 197},	/*  */
-	{'D', 'E', 198},	/*  */
-	{'<', '<', 199},	/*  */
-	{'>', '>', 200},	/*  */
-	{'.', ':', 201},	/*  */
-	{'A', '`', 203},	/*  */
-	{'A', '~', 204},	/*  */
-	{'O', '~', 205},	/*  */
-	{'O', 'E', 206},	/*  */
-	{'o', 'e', 207},	/*  */
-	{'-', '.', 208},	/* - */
-	{'-', '-', 209},	/* - */
-	{'`', '`', 210},	/* " */
-	{'\'', '\'', 211},	/* " */
-	{'`', ' ', 212},	/* ' */
-	{'\'', ' ', 213},	/* ' */
-	{'-', ':', 214},	/*  */
-	{'D', 'I', 215},	/*  */
-	{'y', ':', 216},	/*  */
-	{'Y', ':', 217},	/*  */
-	{'/', '/', 218},	/*  */
-	{'E', '=', 219},	/*  Euro System >=8.5 */
-	{'o', 'x', 219},	/*  Currency System <=8.1*/
-	{'<', ' ', 220},	/*  */
-	{'>', ' ', 221},	/*  */
-	{'f', 'i', 222},	/*  */
-	{'f', 'l', 223},	/*  */
-	{'+', '+', 224},	/*  */
-	{'~', '.', 225},	/*  */
-	{',', ' ', 226},	/*  */
-	{',', ',', 227},	/*  */
-	{'%', '.', 228},	/*  */
-	{'%', '0', 228},	/*  */
-	{'A', '^', 229},	/*  */
-	{'E', '^', 230},	/*  */
-	{'A', '\'', 231},	/*  */
-	{'E', '"', 232},	/*  */
-	{'E', '`', 233},	/*  */
-	{'I', '\'', 234},	/*  */
-	{'I', '^', 235},	/*  */
-	{'I', '"', 236},	/*  */
-	{'I', '`', 237},	/*  */
-	{'O', '\'', 238},	/*  */
-	{'O', '^', 239},	/*  */
-	{'A', 'P', 240},	/*  */
-	{'O', '`', 241},	/*  */
-	{'U', '\'', 242},	/*  */
-	{'U', '^', 243},	/*  */
-	{'U', '`', 244},	/*  */
-	{'i', '.', 245},	/*  */
-	{NUL, NUL, NUL}
-	};
-
-#   else	/* !MACOS_X */
-
-#    ifdef OLD_DIGRAPHS
+#   ifdef OLD_DIGRAPHS
 
 	/*
 	 * digraphs compatible with Vim 5.x
@@ -558,7 +423,7 @@ static digr_T digraphdefault[] =
 	{'y', '"', 255},	/* x XX */
 	{NUL, NUL, NUL}
 	};
-#    else /* OLD_DIGRAPHS */
+#   else /* OLD_DIGRAPHS */
 
 	/*
 	 * digraphs for Unicode from RFC1345
@@ -644,103 +509,155 @@ static digr_T digraphdefault[] =
 	{'P', 'M', 0x9e},
 	{'A', 'C', 0x9f},
 	{'N', 'S', 0xa0},
+#define DG_START_LATIN 0xa1
 	{'!', 'I', 0xa1},
+	{'~', '!', 0xa1},	//  Vim 5.x compatible
 	{'C', 't', 0xa2},
+	{'c', '|', 0xa2},	//  Vim 5.x compatible
 	{'P', 'd', 0xa3},
+	{'$', '$', 0xa3},	//  Vim 5.x compatible
 	{'C', 'u', 0xa4},
+	{'o', 'x', 0xa4},	//  Vim 5.x compatible
 	{'Y', 'e', 0xa5},
+	{'Y', '-', 0xa5},	//  Vim 5.x compatible
 	{'B', 'B', 0xa6},
+	{'|', '|', 0xa6},	//  Vim 5.x compatible
 	{'S', 'E', 0xa7},
 	{'\'', ':', 0xa8},
 	{'C', 'o', 0xa9},
+	{'c', 'O', 0xa9},	//  Vim 5.x compatible
 	{'-', 'a', 0xaa},
 	{'<', '<', 0xab},
 	{'N', 'O', 0xac},
+	{'-', ',', 0xac},	//  Vim 5.x compatible
 	{'-', '-', 0xad},
 	{'R', 'g', 0xae},
 	{'\'', 'm', 0xaf},
+	{'-', '=', 0xaf},	//  Vim 5.x compatible
 	{'D', 'G', 0xb0},
+	{'~', 'o', 0xb0},	//  Vim 5.x compatible
 	{'+', '-', 0xb1},
 	{'2', 'S', 0xb2},
+	{'2', '2', 0xb2},	//  Vim 5.x compatible
 	{'3', 'S', 0xb3},
+	{'3', '3', 0xb3},	//  Vim 5.x compatible
 	{'\'', '\'', 0xb4},
 	{'M', 'y', 0xb5},
 	{'P', 'I', 0xb6},
+	{'p', 'p', 0xb6},	//  Vim 5.x compatible
 	{'.', 'M', 0xb7},
+	{'~', '.', 0xb7},	//  Vim 5.x compatible
 	{'\'', ',', 0xb8},
 	{'1', 'S', 0xb9},
+	{'1', '1', 0xb9},	//  Vim 5.x compatible
 	{'-', 'o', 0xba},
 	{'>', '>', 0xbb},
 	{'1', '4', 0xbc},
 	{'1', '2', 0xbd},
 	{'3', '4', 0xbe},
 	{'?', 'I', 0xbf},
+	{'~', '?', 0xbf},	//  Vim 5.x compatible
 	{'A', '!', 0xc0},
+	{'A', '`', 0xc0},	//  Vim 5.x compatible
 	{'A', '\'', 0xc1},
 	{'A', '>', 0xc2},
+	{'A', '^', 0xc2},	//  Vim 5.x compatible
 	{'A', '?', 0xc3},
+	{'A', '~', 0xc3},	//  Vim 5.x compatible
 	{'A', ':', 0xc4},
+	{'A', '"', 0xc4},	//  Vim 5.x compatible
 	{'A', 'A', 0xc5},
+	{'A', '@', 0xc5},	//  Vim 5.x compatible
 	{'A', 'E', 0xc6},
 	{'C', ',', 0xc7},
 	{'E', '!', 0xc8},
+	{'E', '`', 0xc8},	//  Vim 5.x compatible
 	{'E', '\'', 0xc9},
 	{'E', '>', 0xca},
+	{'E', '^', 0xca},	//  Vim 5.x compatible
 	{'E', ':', 0xcb},
+	{'E', '"', 0xcb},	//  Vim 5.x compatible
 	{'I', '!', 0xcc},
+	{'I', '`', 0xcc},	//  Vim 5.x compatible
 	{'I', '\'', 0xcd},
 	{'I', '>', 0xce},
+	{'I', '^', 0xce},	//  Vim 5.x compatible
 	{'I', ':', 0xcf},
+	{'I', '"', 0xcf},	//  Vim 5.x compatible
 	{'D', '-', 0xd0},
 	{'N', '?', 0xd1},
+	{'N', '~', 0xd1},	//  Vim 5.x compatible
 	{'O', '!', 0xd2},
+	{'O', '`', 0xd2},	//  Vim 5.x compatible
 	{'O', '\'', 0xd3},
 	{'O', '>', 0xd4},
+	{'O', '^', 0xd4},	//  Vim 5.x compatible
 	{'O', '?', 0xd5},
+	{'O', '~', 0xd5},	//  Vim 5.x compatible
 	{'O', ':', 0xd6},
 	{'*', 'X', 0xd7},
+	{'/', '\\', 0xd7},	//  Vim 5.x compatible
 	{'O', '/', 0xd8},
 	{'U', '!', 0xd9},
+	{'U', '`', 0xd9},	//  Vim 5.x compatible
 	{'U', '\'', 0xda},
 	{'U', '>', 0xdb},
+	{'U', '^', 0xdb},	//  Vim 5.x compatible
 	{'U', ':', 0xdc},
 	{'Y', '\'', 0xdd},
 	{'T', 'H', 0xde},
+	{'I', 'p', 0xde},	//  Vim 5.x compatible
 	{'s', 's', 0xdf},
 	{'a', '!', 0xe0},
+	{'a', '`', 0xe0},	//  Vim 5.x compatible
 	{'a', '\'', 0xe1},
 	{'a', '>', 0xe2},
+	{'a', '^', 0xe2},	//  Vim 5.x compatible
 	{'a', '?', 0xe3},
+	{'a', '~', 0xe3},	//  Vim 5.x compatible
 	{'a', ':', 0xe4},
+	{'a', '"', 0xe4},	//  Vim 5.x compatible
 	{'a', 'a', 0xe5},
+	{'a', '@', 0xe5},	//  Vim 5.x compatible
 	{'a', 'e', 0xe6},
 	{'c', ',', 0xe7},
 	{'e', '!', 0xe8},
+	{'e', '`', 0xe8},	//  Vim 5.x compatible
 	{'e', '\'', 0xe9},
 	{'e', '>', 0xea},
+	{'e', '^', 0xea},	//  Vim 5.x compatible
 	{'e', ':', 0xeb},
+	{'e', '"', 0xeb},	//  Vim 5.x compatible
 	{'i', '!', 0xec},
+	{'i', '`', 0xec},	//  Vim 5.x compatible
 	{'i', '\'', 0xed},
 	{'i', '>', 0xee},
+	{'i', '^', 0xee},	//  Vim 5.x compatible
 	{'i', ':', 0xef},
 	{'d', '-', 0xf0},
 	{'n', '?', 0xf1},
+	{'n', '~', 0xf1},	//  Vim 5.x compatible
 	{'o', '!', 0xf2},
+	{'o', '`', 0xf2},	//  Vim 5.x compatible
 	{'o', '\'', 0xf3},
 	{'o', '>', 0xf4},
+	{'o', '^', 0xf4},	//  Vim 5.x compatible
 	{'o', '?', 0xf5},
+	{'o', '~', 0xf5},	//  Vim 5.x compatible
 	{'o', ':', 0xf6},
 	{'-', ':', 0xf7},
 	{'o', '/', 0xf8},
 	{'u', '!', 0xf9},
+	{'u', '`', 0xf9},	//  Vim 5.x compatible
 	{'u', '\'', 0xfa},
 	{'u', '>', 0xfb},
+	{'u', '^', 0xfb},	//  Vim 5.x compatible
 	{'u', ':', 0xfc},
 	{'y', '\'', 0xfd},
 	{'t', 'h', 0xfe},
 	{'y', ':', 0xff},
+	{'y', '"', 0xff},	// x XX  Vim 5.x compatible
 
-#      ifdef FEAT_MBYTE
 #	define USE_UNICODE_DIGRAPHS
 
 	{'A', '-', 0x0100},
@@ -916,6 +833,7 @@ static digr_T digraphdefault[] =
 	{'\'', '0', 0x02da},
 	{'\'', ';', 0x02db},
 	{'\'', '"', 0x02dd},
+#define DG_START_GREEK 0x0386
 	{'A', '%', 0x0386},
 	{'E', '%', 0x0388},
 	{'Y', '%', 0x0389},
@@ -997,6 +915,7 @@ static digr_T digraphdefault[] =
 	{'p', '3', 0x03e1},
 	{'\'', '%', 0x03f4},
 	{'j', '3', 0x03f5},
+#define DG_START_CYRILLIC 0x0401
 	{'I', 'O', 0x0401},
 	{'D', '%', 0x0402},
 	{'G', '%', 0x0403},
@@ -1101,6 +1020,7 @@ static digr_T digraphdefault[] =
 	{'c', '3', 0x0481},
 	{'G', '3', 0x0490},
 	{'g', '3', 0x0491},
+#define DG_START_HEBREW 0x05d0
 	{'A', '+', 0x05d0},
 	{'B', '+', 0x05d1},
 	{'G', '+', 0x05d2},
@@ -1128,6 +1048,7 @@ static digr_T digraphdefault[] =
 	{'R', '+', 0x05e8},
 	{'S', 'h', 0x05e9},
 	{'T', '+', 0x05ea},
+#define DG_START_ARABIC 0x060c
 	{',', '+', 0x060c},
 	{';', '+', 0x061b},
 	{'?', '+', 0x061f},
@@ -1190,6 +1111,7 @@ static digr_T digraphdefault[] =
 	{'7', 'a', 0x06f7},
 	{'8', 'a', 0x06f8},
 	{'9', 'a', 0x06f9},
+#define DG_START_LATIN_EXTENDED 0x1e02
 	{'B', '.', 0x1e02},
 	{'b', '.', 0x1e03},
 	{'B', '_', 0x1e06},
@@ -1241,7 +1163,9 @@ static digr_T digraphdefault[] =
 	{'V', '?', 0x1e7c},
 	{'v', '?', 0x1e7d},
 	{'W', '!', 0x1e80},
+	{'W', '`', 0x1e80}, // extra alternative, easier to remember
 	{'w', '!', 0x1e81},
+	{'w', '`', 0x1e81}, // extra alternative, easier to remember
 	{'W', '\'', 0x1e82},
 	{'w', '\'', 0x1e83},
 	{'W', ':', 0x1e84},
@@ -1275,11 +1199,14 @@ static digr_T digraphdefault[] =
 	{'U', '2', 0x1ee6},
 	{'u', '2', 0x1ee7},
 	{'Y', '!', 0x1ef2},
+	{'Y', '`', 0x1ef2}, // extra alternative, easier to remember
 	{'y', '!', 0x1ef3},
+	{'y', '`', 0x1ef3}, // extra alternative, easier to remember
 	{'Y', '2', 0x1ef6},
 	{'y', '2', 0x1ef7},
 	{'Y', '?', 0x1ef8},
 	{'y', '?', 0x1ef9},
+#define DG_START_GREEK_EXTENDED 0x1f00
 	{';', '\'', 0x1f00},
 	{',', '\'', 0x1f01},
 	{';', '!', 0x1f02},
@@ -1288,6 +1215,7 @@ static digr_T digraphdefault[] =
 	{'?', ',', 0x1f05},
 	{'!', ':', 0x1f06},
 	{'?', ':', 0x1f07},
+#define DG_START_PUNCTUATION 0x2002
 	{'1', 'N', 0x2002},
 	{'1', 'M', 0x2003},
 	{'3', 'M', 0x2004},
@@ -1326,6 +1254,7 @@ static digr_T digraphdefault[] =
 	{':', 'X', 0x203b},
 	{'\'', '-', 0x203e},
 	{'/', 'f', 0x2044},
+#define DG_START_SUB_SUPER 0x2070
 	{'0', 'S', 0x2070},
 	{'4', 'S', 0x2074},
 	{'5', 'S', 0x2075},
@@ -1354,6 +1283,7 @@ static digr_T digraphdefault[] =
 	{'=', 's', 0x208c},
 	{'(', 's', 0x208d},
 	{')', 's', 0x208e},
+#define DG_START_CURRENCY 0x20a4
 	{'L', 'i', 0x20a4},
 	{'P', 't', 0x20a7},
 	{'W', '=', 0x20a9},
@@ -1361,6 +1291,7 @@ static digr_T digraphdefault[] =
 	{'E', 'u', 0x20ac}, /* euro */
 	{'=', 'R', 0x20bd}, /* rouble */
 	{'=', 'P', 0x20bd}, /* rouble */
+#define DG_START_OTHER1 0x2103
 	{'o', 'C', 0x2103},
 	{'c', 'o', 0x2105},
 	{'o', 'F', 0x2109},
@@ -1383,6 +1314,7 @@ static digr_T digraphdefault[] =
 	{'3', '8', 0x215c},
 	{'5', '8', 0x215d},
 	{'7', '8', 0x215e},
+#define DG_START_ROMAN 0x2160
 	{'1', 'R', 0x2160},
 	{'2', 'R', 0x2161},
 	{'3', 'R', 0x2162},
@@ -1407,6 +1339,7 @@ static digr_T digraphdefault[] =
 	{'a', 'r', 0x2179},
 	{'b', 'r', 0x217a},
 	{'c', 'r', 0x217b},
+#define DG_START_ARROWS 0x2190
 	{'<', '-', 0x2190},
 	{'-', '!', 0x2191},
 	{'-', '>', 0x2192},
@@ -1416,6 +1349,7 @@ static digr_T digraphdefault[] =
 	{'<', '=', 0x21d0},
 	{'=', '>', 0x21d2},
 	{'=', '=', 0x21d4},
+#define DG_START_MATH 0x2200
 	{'F', 'A', 0x2200},
 	{'d', 'P', 0x2202},
 	{'T', 'E', 0x2203},
@@ -1473,6 +1407,7 @@ static digr_T digraphdefault[] =
 	{'.', 'P', 0x22c5},
 	{':', '3', 0x22ee},
 	{'.', '3', 0x22ef},
+#define DG_START_TECHNICAL 0x2302
 	{'E', 'h', 0x2302},
 	{'<', '7', 0x2308},
 	{'>', '7', 0x2309},
@@ -1485,6 +1420,7 @@ static digr_T digraphdefault[] =
 	{'I', 'l', 0x2321},
 	{'<', '/', 0x2329},
 	{'/', '>', 0x232a},
+#define DG_START_OTHER2 0x2423
 	{'V', 's', 0x2423},
 	{'1', 'h', 0x2440},
 	{'3', 'h', 0x2441},
@@ -1503,6 +1439,7 @@ static digr_T digraphdefault[] =
 	{'7', '.', 0x248e},
 	{'8', '.', 0x248f},
 	{'9', '.', 0x2490},
+#define DG_START_DRAWING 0x2500
 	{'h', 'h', 0x2500},
 	{'H', 'H', 0x2501},
 	{'v', 'v', 0x2502},
@@ -1553,6 +1490,7 @@ static digr_T digraphdefault[] =
 	{'V', 'H', 0x254b},
 	{'F', 'D', 0x2571},
 	{'B', 'D', 0x2572},
+#define DG_START_BLOCK 0x2580
 	{'T', 'B', 0x2580},
 	{'L', 'B', 0x2584},
 	{'F', 'B', 0x2588},
@@ -1561,6 +1499,7 @@ static digr_T digraphdefault[] =
 	{'.', 'S', 0x2591},
 	{':', 'S', 0x2592},
 	{'?', 'S', 0x2593},
+#define DG_START_SHAPES 0x25a0
 	{'f', 'S', 0x25a0},
 	{'O', 'S', 0x25a1},
 	{'R', 'O', 0x25a2},
@@ -1594,6 +1533,7 @@ static digr_T digraphdefault[] =
 	{'I', 'c', 0x25d9},
 	{'F', 'd', 0x25e2},
 	{'B', 'd', 0x25e3},
+#define DG_START_SYMBOLS 0x2605
 	{'*', '2', 0x2605},
 	{'*', '1', 0x2606},
 	{'<', 'H', 0x261c},
@@ -1613,9 +1553,11 @@ static digr_T digraphdefault[] =
 	{'M', 'b', 0x266d},
 	{'M', 'x', 0x266e},
 	{'M', 'X', 0x266f},
+#define DG_START_DINGBATS 0x2713
 	{'O', 'K', 0x2713},
 	{'X', 'X', 0x2717},
 	{'-', 'X', 0x2720},
+#define DG_START_CJK_SYMBOLS 0x3000
 	{'I', 'S', 0x3000},
 	{',', '_', 0x3001},
 	{'.', '_', 0x3002},
@@ -1639,6 +1581,7 @@ static digr_T digraphdefault[] =
 	{'(', 'I', 0x3016},
 	{')', 'I', 0x3017},
 	{'-', '?', 0x301c},
+#define DG_START_HIRAGANA 0x3041
 	{'A', '5', 0x3041},
 	{'a', '5', 0x3042},
 	{'I', '5', 0x3043},
@@ -1727,6 +1670,7 @@ static digr_T digraphdefault[] =
 	{'0', '5', 0x309c},
 	{'*', '5', 0x309d},
 	{'+', '5', 0x309e},
+#define DG_START_KATAKANA 0x30a1
 	{'a', '6', 0x30a1},
 	{'A', '6', 0x30a2},
 	{'i', '6', 0x30a3},
@@ -1821,6 +1765,7 @@ static digr_T digraphdefault[] =
 	{'-', '6', 0x30fc},
 	{'*', '6', 0x30fd},
 	{'+', '6', 0x30fe},
+#define DG_START_BOPOMOFO 0x3105
 	{'b', '4', 0x3105},
 	{'p', '4', 0x3106},
 	{'m', '4', 0x3107},
@@ -1860,6 +1805,7 @@ static digr_T digraphdefault[] =
 	{'v', '4', 0x312a},
 	{'n', 'G', 0x312b},
 	{'g', 'n', 0x312c},
+#define DG_START_OTHER3 0x3220
 	{'1', 'c', 0x3220},
 	{'2', 'c', 0x3221},
 	{'3', 'c', 0x3222},
@@ -1877,74 +1823,10 @@ static digr_T digraphdefault[] =
 	{'f', 't', 0xfb05},
 	{'s', 't', 0xfb06},
 
-	/* extra alternatives, easier to remember */
-	{'W', '`', 0x1e80},
-	{'w', '`', 0x1e81},
-	{'Y', '`', 0x1ef2},
-	{'y', '`', 0x1ef3},
-
-#      endif /* FEAT_MBYTE */
-
-	/* Vim 5.x compatible digraphs that don't conflict with the above */
-	{'~', '!', 161},	/*  */
-	{'c', '|', 162},	/*  */
-	{'$', '$', 163},	/*  */
-	{'o', 'x', 164},	/*  - currency symbol in ISO 8859-1 */
-	{'Y', '-', 165},	/*  */
-	{'|', '|', 166},	/*  */
-	{'c', 'O', 169},	/*  */
-	{'-', ',', 172},	/*  */
-	{'-', '=', 175},	/*  */
-	{'~', 'o', 176},	/*  */
-	{'2', '2', 178},	/*  */
-	{'3', '3', 179},	/*  */
-	{'p', 'p', 182},	/*  */
-	{'~', '.', 183},	/*  */
-	{'1', '1', 185},	/*  */
-	{'~', '?', 191},	/*  */
-	{'A', '`', 192},	/*  */
-	{'A', '^', 194},	/*  */
-	{'A', '~', 195},	/*  */
-	{'A', '"', 196},	/*  */
-	{'A', '@', 197},	/*  */
-	{'E', '`', 200},	/*  */
-	{'E', '^', 202},	/*  */
-	{'E', '"', 203},	/*  */
-	{'I', '`', 204},	/*  */
-	{'I', '^', 206},	/*  */
-	{'I', '"', 207},	/*  */
-	{'N', '~', 209},	/*  */
-	{'O', '`', 210},	/*  */
-	{'O', '^', 212},	/*  */
-	{'O', '~', 213},	/*  */
-	{'/', '\\', 215},	/*  - multiplication symbol in ISO 8859-1 */
-	{'U', '`', 217},	/*  */
-	{'U', '^', 219},	/*  */
-	{'I', 'p', 222},	/*  */
-	{'a', '`', 224},	/*  */
-	{'a', '^', 226},	/*  */
-	{'a', '~', 227},	/*  */
-	{'a', '"', 228},	/*  */
-	{'a', '@', 229},	/*  */
-	{'e', '`', 232},	/*  */
-	{'e', '^', 234},	/*  */
-	{'e', '"', 235},	/*  */
-	{'i', '`', 236},	/*  */
-	{'i', '^', 238},	/*  */
-	{'n', '~', 241},	/*  */
-	{'o', '`', 242},	/*  */
-	{'o', '^', 244},	/*  */
-	{'o', '~', 245},	/*  */
-	{'u', '`', 249},	/*  */
-	{'u', '^', 251},	/*  */
-	{'y', '"', 255},	/* x XX */
-
 	{NUL, NUL, NUL}
        };
 
-#    endif /* OLD_DIGRAPHS */
-
-#   endif /* Macintosh */
+#   endif /* OLD_DIGRAPHS */
 #  endif /* EBCDIC */
 # endif    /* !HPUX_DIGRAPHS */
 #endif	/* !__MINT__ */
@@ -1987,7 +1869,7 @@ get_digraph_for_char(int val_arg)
     digr_T	*dp;
     static      char_u      r[3];
 
-#if defined(FEAT_MBYTE) && defined(USE_UNICODE_DIGRAPHS)
+#if defined(USE_UNICODE_DIGRAPHS)
     if (!enc_utf8)
     {
 	char_u	    buf[6], *to;
@@ -2121,8 +2003,7 @@ getexactdigraph(int char1, int char2, int meta_char)
 	    ++dp;
 	}
     }
-#ifdef FEAT_MBYTE
-# ifdef USE_UNICODE_DIGRAPHS
+#ifdef USE_UNICODE_DIGRAPHS
     if (retval != 0 && !enc_utf8)
     {
 	char_u	    buf[6], *to;
@@ -2146,12 +2027,11 @@ getexactdigraph(int char1, int char2, int meta_char)
 	    (void)convert_setup(&vc, NULL, NULL);
 	}
     }
-# endif
+#endif
 
     /* Ignore multi-byte characters when not in multi-byte mode. */
     if (!has_mbyte && retval > 0xff)
 	retval = 0;
-#endif
 
     if (retval == 0)		/* digraph deleted or not found */
     {
@@ -2198,18 +2078,18 @@ putdigraph(char_u *str)
 	char2 = *str++;
 	if (char2 == 0)
 	{
-	    EMSG(_(e_invarg));
+	    emsg(_(e_invarg));
 	    return;
 	}
 	if (char1 == ESC || char2 == ESC)
 	{
-	    EMSG(_("E104: Escape not allowed in digraph"));
+	    emsg(_("E104: Escape not allowed in digraph"));
 	    return;
 	}
 	str = skipwhite(str);
 	if (!VIM_ISDIGIT(*str))
 	{
-	    EMSG(_(e_number_exp));
+	    emsg(_(e_number_exp));
 	    return;
 	}
 	n = getdigits(&str);
@@ -2241,18 +2121,30 @@ putdigraph(char_u *str)
     }
 }
 
+#if defined(USE_UNICODE_DIGRAPHS)
+    static void
+digraph_header(char *msg)
+{
+    if (msg_col > 0)
+	msg_putchar('\n');
+    msg_outtrans_attr((char_u *)msg, HL_ATTR(HLF_CM));
+    msg_putchar('\n');
+}
+#endif
+
     void
-listdigraphs(void)
+listdigraphs(int use_headers)
 {
     int		i;
     digr_T	*dp;
+    result_T	previous = 0;
 
     msg_putchar('\n');
 
     dp = digraphdefault;
     for (i = 0; dp->char1 != NUL && !got_int; ++i)
     {
-#if defined(USE_UNICODE_DIGRAPHS) && defined(FEAT_MBYTE)
+#if defined(USE_UNICODE_DIGRAPHS)
 	digr_T tmp;
 
 	/* May need to convert the result to 'encoding'. */
@@ -2261,15 +2153,12 @@ listdigraphs(void)
 	tmp.result = getexactdigraph(tmp.char1, tmp.char2, FALSE);
 	if (tmp.result != 0 && tmp.result != tmp.char2
 					  && (has_mbyte || tmp.result <= 255))
-	    printdigraph(&tmp);
+	    printdigraph(&tmp, use_headers ? &previous : NULL);
 #else
 
 	if (getexactdigraph(dp->char1, dp->char2, FALSE) == dp->result
-# ifdef FEAT_MBYTE
-		&& (has_mbyte || dp->result <= 255)
-# endif
-		)
-	    printdigraph(dp);
+		&& (has_mbyte || dp->result <= 255))
+	    printdigraph(dp, use_headers ? &previous : NULL);
 #endif
 	++dp;
 	ui_breakcheck();
@@ -2278,7 +2167,12 @@ listdigraphs(void)
     dp = (digr_T *)user_digraphs.ga_data;
     for (i = 0; i < user_digraphs.ga_len && !got_int; ++i)
     {
-	printdigraph(dp);
+#if defined(USE_UNICODE_DIGRAPHS)
+	if (previous >= 0 && use_headers)
+	    digraph_header(_("Custom"));
+	previous = -1;
+#endif
+	printdigraph(dp, NULL);
 	ui_breakcheck();
 	++dp;
     }
@@ -2286,25 +2180,70 @@ listdigraphs(void)
 			       wrong, in which case we messed up ScreenLines */
 }
 
+struct dg_header_entry {
+    int	    dg_start;
+    char    *dg_header;
+} header_table[] = {
+    {DG_START_LATIN, N_("Latin supplement")},
+    {DG_START_GREEK, N_("Greek and Coptic")},
+    {DG_START_CYRILLIC, N_("Cyrillic")},
+    {DG_START_HEBREW, N_("Hebrew")},
+    {DG_START_ARABIC, N_("Arabic")},
+    {DG_START_LATIN_EXTENDED, N_("Latin extended")},
+    {DG_START_GREEK_EXTENDED, N_("Greek extended")},
+    {DG_START_PUNCTUATION, N_("Punctuation")},
+    {DG_START_SUB_SUPER, N_("Super- and subscripts")},
+    {DG_START_CURRENCY, N_("Currency")},
+    {DG_START_OTHER1, N_("Other")},
+    {DG_START_ROMAN, N_("Roman numbers")},
+    {DG_START_ARROWS, N_("Arrows")},
+    {DG_START_MATH, N_("Mathematical operators")},
+    {DG_START_TECHNICAL, N_("Technical")},
+    {DG_START_OTHER2, N_("Other")},
+    {DG_START_DRAWING, N_("Box drawing")},
+    {DG_START_BLOCK, N_("Block elements")},
+    {DG_START_SHAPES, N_("Geometric shapes")},
+    {DG_START_SYMBOLS, N_("Symbols")},
+    {DG_START_DINGBATS, N_("Dingbats")},
+    {DG_START_CJK_SYMBOLS, N_("CJK symbols and punctuation")},
+    {DG_START_HIRAGANA, N_("Hiragana")},
+    {DG_START_KATAKANA, N_("Katakana")},
+    {DG_START_BOPOMOFO, N_("Bopomofo")},
+    {DG_START_OTHER3, N_("Other")},
+    {0xfffffff, NULL},
+};
+
     static void
-printdigraph(digr_T *dp)
+printdigraph(digr_T *dp, result_T *previous)
 {
     char_u	buf[30];
     char_u	*p;
 
     int		list_width;
 
-    if ((dy_flags & DY_UHEX)
-#ifdef FEAT_MBYTE
-	    || has_mbyte
-#endif
-	    )
+    if ((dy_flags & DY_UHEX) || has_mbyte)
 	list_width = 13;
     else
 	list_width = 11;
 
     if (dp->result != 0)
     {
+#if defined(USE_UNICODE_DIGRAPHS)
+	if (previous != NULL)
+	{
+	    int i;
+
+	    for (i = 0; header_table[i].dg_header != NULL; ++i)
+		if (*previous < header_table[i].dg_start
+			&& dp->result >= header_table[i].dg_start
+			&& dp->result < header_table[i + 1].dg_start)
+		{
+		    digraph_header(_(header_table[i].dg_header));
+		    break;
+		}
+	    *previous = dp->result;
+	}
+#endif
 	if (msg_col > Columns - list_width)
 	    msg_putchar('\n');
 	if (msg_col)
@@ -2315,7 +2254,9 @@ printdigraph(digr_T *dp)
 	*p++ = dp->char1;
 	*p++ = dp->char2;
 	*p++ = ' ';
-#ifdef FEAT_MBYTE
+	*p = NUL;
+	msg_outtrans(buf);
+	p = buf;
 	if (has_mbyte)
 	{
 	    /* add a space to draw a composing char on */
@@ -2324,8 +2265,10 @@ printdigraph(digr_T *dp)
 	    p += (*mb_char2bytes)(dp->result, p);
 	}
 	else
-#endif
 	    *p++ = (char_u)dp->result;
+	*p = NUL;
+	msg_outtrans_attr(buf, HL_ATTR(HLF_8));
+	p = buf;
 	if (char2cells(dp->result) == 1)
 	    *p++ = ' ';
 	vim_snprintf((char *)p, sizeof(buf) - (p - buf), " %3d", dp->result);
@@ -2354,7 +2297,7 @@ static void keymap_unload(void);
  * used when setting the option, not later when the value has already been
  * checked.
  */
-    char_u *
+    char *
 keymap_init(void)
 {
     curbuf->b_kmap_state &= ~KEYMAP_INIT;
@@ -2373,21 +2316,15 @@ keymap_init(void)
 
 	/* Source the keymap file.  It will contain a ":loadkeymap" command
 	 * which will call ex_loadkeymap() below. */
-	buflen = STRLEN(curbuf->b_p_keymap)
-# ifdef FEAT_MBYTE
-					   + STRLEN(p_enc)
-# endif
-						       + 14;
-	buf = alloc((unsigned)buflen);
+	buflen = STRLEN(curbuf->b_p_keymap) + STRLEN(p_enc) + 14;
+	buf = alloc(buflen);
 	if (buf == NULL)
 	    return e_outofmem;
 
-# ifdef FEAT_MBYTE
 	/* try finding "keymap/'keymap'_'encoding'.vim"  in 'runtimepath' */
 	vim_snprintf((char *)buf, buflen, "keymap/%s_%s.vim",
 						   curbuf->b_p_keymap, p_enc);
 	if (source_runtime(buf, 0) == FAIL)
-# endif
 	{
 	    /* try finding "keymap/'keymap'.vim" in 'runtimepath'  */
 	    vim_snprintf((char *)buf, buflen, "keymap/%s.vim",
@@ -2395,7 +2332,7 @@ keymap_init(void)
 	    if (source_runtime(buf, 0) == FAIL)
 	    {
 		vim_free(buf);
-		return (char_u *)N_("E544: Keymap file not found");
+		return N_("E544: Keymap file not found");
 	    }
 	}
 	vim_free(buf);
@@ -2421,7 +2358,7 @@ ex_loadkeymap(exarg_T *eap)
 
     if (!getline_equal(eap->getline, eap->cookie, getsourceline))
     {
-	EMSG(_("E105: Using :loadkeymap not in a sourced file"));
+	emsg(_("E105: Using :loadkeymap not in a sourced file"));
 	return;
     }
 
@@ -2460,7 +2397,7 @@ ex_loadkeymap(exarg_T *eap)
 		    || *kp->from == NUL || *kp->to == NUL)
 	    {
 		if (kp->to != NULL && *kp->to == NUL)
-		    EMSG(_("E791: Empty keymap entry"));
+		    emsg(_("E791: Empty keymap entry"));
 		vim_free(kp->from);
 		vim_free(kp->to);
 	    }
