@@ -1484,7 +1484,7 @@ clip_invert_area(
  */
     static void
 clip_invert_rectangle(
-	Clipboard_T	*cbd,
+	Clipboard_T	*cbd UNUSED,
 	int		row_arg,
 	int		col_arg,
 	int		height_arg,
@@ -1498,7 +1498,7 @@ clip_invert_rectangle(
 
 #ifdef FEAT_TEXT_PROP
     // this goes on top of all popup windows
-    screen_zindex = 32000;
+    screen_zindex = CLIP_ZINDEX;
 
     if (col < cbd->min_col)
     {
@@ -2929,7 +2929,6 @@ jump_to_mouse(
 #endif
 #ifdef FEAT_TEXT_PROP
     static int   in_popup_win = FALSE;
-    static win_T *popup_dragwin = NULL;
 #endif
     static int	prev_row = -1;
     static int	prev_col = -1;
